#line 1 "F:/MikroC/ModbusSlave/ModbusSlave.c"
#line 1 "f:/mikroc/modbusslave/modbusslave.h"
#line 1 "f:/mikroc/modbusslave/types.h"
#line 14 "f:/mikroc/modbusslave/types.h"
 typedef unsigned long ulong;


 typedef unsigned int uint;


 typedef unsigned short ushort;


 typedef unsigned char byte;


 typedef enum ebool { FALSE, TRUE } boolean;
#line 89 "f:/mikroc/modbusslave/modbusslave.h"
 typedef struct {
 byte bytHi;
 byte bytLo;
 } hiloDef;

 typedef enum eBaudRates {
 BAUD_1200 = 12,
 BAUD_2400 = 24,
 BAUD_4800 = 48,
 BAUD_9600 = 96,
 BAUD_19200 = 192,
 BAUD_38400 = 384,
 BAUD_57600 = 576,
 BAUD_115200 = 1152
 } baudRate;

 typedef enum eModbusFunctions {
 READ_COILS = 1,
 READ_STATUS_INPUTS = 2,
 READ_HOLDING_REGISTERS = 3,
 READ_INPUT_REGISTERS = 4,
 FORCE_SINGLE_COIL = 5,
 PRESET_SINGLE_REGISTER = 6,
 FORCE_MULTIPLE_COILS = 15,
 PRESET_MULTIPLE_REGISTERS = 16
 } mbFunction;

 typedef enum eModbusExceptions {
 NO_EXCEPTION = 0,
 ILLEGAL_FUNCTION = 1,
 ILLEGAL_DATA_ADDRESS = 2,
 ILLEGAL_DATA_VALUE = 3,
 SLAVE_DEVICE_FAILURE = 4,
 ACKNOWLEDGE = 5,
 SLAVE_DEVICE_BUSY = 6,
 NEGATIVE_ACKNOWLEDGE = 7,
 MEMORY_PARITY_ERROR = 8
 } mbException;

 typedef struct _discrete {

 uint uintAddress;

 ushort* pByte;

 byte bBit;

 struct _discrete* pNext;
 } discreteDef;

 typedef discreteDef coilDef;
 typedef discreteDef statusDef;


 typedef struct {
 byte bytSlave;
 byte bytFunction;
 hiloDef addr;
 hiloDef points;
 hiloDef crc;
 } reqDataDef;

 typedef reqDataDef reqCoilsDef;
 typedef reqDataDef reqStatusDef;
 typedef reqDataDef reqHoldReqDef;
 typedef reqDataDef reqInputsDef;

 typedef struct _register {

 uint uintAddress;

 ushort* pWord;

 struct _register* pNext;
 } registerDef;

 typedef registerDef inputDef;
 typedef registerDef holdingDef;

 boolean addCoil(coilDef* pCoil,
 uint uintAddress, ushort* pByte, byte bBit);
 boolean addHoldingReg(holdingDef* pHolding,
 uint uintAddress, ushort* pWord);
 boolean addInputReg(inputDef* pInput,
 uint uintAddress, ushort* pWord);
 boolean addStatusBit(statusDef* pStatus,
 uint uintAddress, ushort* pByte, byte bBit);
 void decodePacket(void);
 int modbusSlave(baudRate eBaud, const byte bytAddress);
#line 25 "F:/MikroC/ModbusSlave/ModbusSlave.c"
static coilDef* pLastCoil =  ((void*)0) ;
static statusDef* pLastStatusBit =  ((void*)0) ;
static inputDef* pLastInput =  ((void*)0) ;
static holdingDef* pLastHoldingReg =  ((void*)0) ;

static coilDef* pCoils =  ((void*)0) ;
static statusDef* pStatusBits =  ((void*)0) ;
static inputDef* pInputs =  ((void*)0) ;
static holdingDef* pHoldingRegs =  ((void*)0) ;

static boolean blnValidStart, blnValidEnd;

static mbException exceptionCode;

static byte bytSlaveAddress;

static byte bytStartHi, bytStartLo;

static byte bytTotalHi, bytTotalLo;

static byte bytIndex = 0;

static uint uintRxGapSetPt;

static uint uintRxGap;

static boolean blnCheckBuffer = FALSE;

union {
 byte arybytes[ 255 ];
 reqCoilsDef reqCoils;
 reqStatusDef reqStatus;
 reqHoldReqDef reqHoldReg;
 reqInputsDef reqInputs;
} request;

union {
 byte arybytes[ 255 ];
} reply;
#line 79 "F:/MikroC/ModbusSlave/ModbusSlave.c"
static boolean addDiscrete(discreteDef* pDiscrete,
 uint uintAddress,
 ushort* pByte, byte bBit,
 discreteDef** pLinkList,
 discreteDef** pLast) {
 if ( !(pDiscrete && uintAddress && pByte &&
 bBit >= 0 && bBit < 8 && pLinkList && pLast) ) {
 return FALSE;
 }
 if ( *pLinkList ==  ((void*)0)  ) {

 *pLinkList = *pLast = pDiscrete;
 } else {

 (*pLast)->pNext = pDiscrete;

 *pLast = pDiscrete;
 }

 pDiscrete->uintAddress = uintAddress;
 pDiscrete->pByte = pByte;
 pDiscrete->bBit = bBit;
 pDiscrete->pNext =  ((void*)0) ;
}
#line 117 "F:/MikroC/ModbusSlave/ModbusSlave.c"
static boolean addRegister(registerDef* pRegister,
 uint uintAddress,
 ushort* pWord,
 registerDef** pLinkList,
 registerDef** pLast) {
 if ( !(pRegister && uintAddress && pWord && pLinkList && pLast) ) {
 return FALSE;
 }
 if ( *pLinkList ==  ((void*)0)  ) {

 *pLinkList = *pLast = pRegister;
 } else {

 (*pLast)->pNext = pRegister;

 *pLast = pRegister;
 }

 pRegister->uintAddress = uintAddress;
 pRegister->pWord = pWord;
 pRegister->pNext =  ((void*)0) ;
}
#line 150 "F:/MikroC/ModbusSlave/ModbusSlave.c"
static long calcCRC(byte* pMsg, uint uintLength) {
 long lngCRC = 0xffff;
 uint i, j;
 for( i=0; i<uintLength; i++ ) {
 lngCRC ^= (pMsg[i] & 0xff);

 for( j=0; j<8; j++ ) {
 if ( lngCRC & 0x01 ) {
 lngCRC >>= 0x01;
 lngCRC ^= 0xa001;
 } else {
 lngCRC >>= 0x01;
 }
 }
 }
 return lngCRC;
}
#line 178 "F:/MikroC/ModbusSlave/ModbusSlave.c"
static void coilState(uint uintAddr, boolean eState) {

 discreteDef* pNode;
 byte bytMask;
 for( pNode=pCoils; pNode!= ((void*)0) ; pNode=pNode->pNext ) {
 if ( pNode->uintAddress != uintAddr ) {
 continue;
 }
 bytMask = (1 << pNode->bBit);

 if ( eState == TRUE ) {

 *pNode->pByte |= bytMask;
 } else {

 *pNode->pByte &= (0xffff ^ bytMask);
 }
 break;
 }
}
#line 211 "F:/MikroC/ModbusSlave/ModbusSlave.c"
static uint packBits(discreteDef* pLL, byte* pBuffer,
 uint uintStart, uint uintEnd) {

 uint uintBytes = 0, uintBit = 0;
 discreteDef* pNode;
 byte bytSet;
 for( pNode=pLL; pNode!= ((void*)0) ; pNode=pNode->pNext ) {
 if ( pNode->uintAddress >= uintStart && pNode->uintAddress <= uintEnd ) {
 bytSet = (*pNode->pByte & (1 << pNode->bBit)) ? 1 : 0;

 if ( bytSet ) {
 pBuffer[uintBytes] |= (bytSet << uintBit);
 }
 if ( ++uintBit > 7 ) {
 uintBit = 0;
 uintBytes++;
 }
 }
 }
 return uintBytes + 1;
}
#line 245 "F:/MikroC/ModbusSlave/ModbusSlave.c"
static uint packRegisters(registerDef* pLL, byte* pBuffer,
 uint uintStart, uint uintEnd) {

 uint uintBytes = 0;
 registerDef* pNode;
 for( pNode=pLL; pNode!= ((void*)0) ; pNode=pNode->pNext ) {
 if ( pNode->uintAddress >= uintStart && pNode->uintAddress <= uintEnd ) {
 pBuffer[uintBytes++] = (*pNode->pWord >> 0x08);
 pBuffer[uintBytes++] = (*pNode->pWord & 0xff);
 }
 }
 return uintBytes;
}
#line 270 "F:/MikroC/ModbusSlave/ModbusSlave.c"
static void setRegister(uint uintAddr, byte bytHi, byte bytLo) {

 registerDef* pNode;
 for( pNode=pHoldingRegs; pNode!= ((void*)0) ; pNode=pNode->pNext ) {
 if ( pNode->uintAddress != uintAddr ) {
 continue;
 }
 *pNode->pWord = ((int)bytHi << 0x08) | (int)bytLo;
 break;
 }
}
#line 291 "F:/MikroC/ModbusSlave/ModbusSlave.c"
static void startTimeout(void) {

 TMR0IE_bit = 0;

 TMR0H =  0xfe ;
 TMR0L =  0x70 ;

 uintRxGap = 0;

 TMR0IF_bit = 0;

 TMR0IE_bit = 1;
}
#line 316 "F:/MikroC/ModbusSlave/ModbusSlave.c"
static boolean validAddresses(uint uintAddress, uint uintStart, uint uintEnd) {
 if ( blnValidStart == FALSE && uintStart == uintAddress ) {
 blnValidStart = TRUE;
 }
 if ( blnValidEnd == FALSE && uintEnd == uintAddress ) {
 blnValidEnd = TRUE;
 }
 return (blnValidStart == TRUE && blnValidEnd == TRUE);
}
#line 337 "F:/MikroC/ModbusSlave/ModbusSlave.c"
static boolean validDiscretes(discreteDef* pLL, uint uintStart, uint uintEnd) {
 discreteDef* pNode;
 blnValidStart = blnValidEnd = FALSE;
 for( pNode=pLL; pNode!= ((void*)0) ; pNode=pNode->pNext ) {
 if ( validAddresses(pNode->uintAddress, uintStart, uintEnd) == TRUE ) {
 return TRUE;
 }
 }
 return FALSE;
}
#line 359 "F:/MikroC/ModbusSlave/ModbusSlave.c"
static boolean validRegisters(registerDef* pLL, uint uintStart, uint uintEnd) {
 registerDef* pNode;
 blnValidStart = blnValidEnd = FALSE;
 for( pNode=pLL; pNode!= ((void*)0) ; pNode=pNode->pNext ) {
 if ( validAddresses(pNode->uintAddress, uintStart, uintEnd) == TRUE ) {
 return TRUE;
 }
 }
 return FALSE;
}
#line 382 "F:/MikroC/ModbusSlave/ModbusSlave.c"
boolean addCoil(coilDef* pCoil, uint uintAddress, ushort* pByte, byte bBit) {
 return addDiscrete(pCoil, uintAddress, pByte, bBit, &pCoils, &pLastCoil);
}
#line 397 "F:/MikroC/ModbusSlave/ModbusSlave.c"
boolean addHoldingReg(holdingDef* pHolding, uint uintAddress, ushort* pWord) {
 return addRegister(pHolding, uintAddress, pWord, &pHoldingRegs,
 &pLastHoldingReg);
}
#line 413 "F:/MikroC/ModbusSlave/ModbusSlave.c"
boolean addInputReg(inputDef* pInput, uint uintAddress, ushort* pWord) {
 return addRegister(pInput, uintAddress, pWord, &pInputs, &pLastInput);
}
#line 429 "F:/MikroC/ModbusSlave/ModbusSlave.c"
boolean addStatusBit(statusDef* pStatus, uint uintAddress, ushort* pByte,
 byte bBit) {
 return addDiscrete(pStatus, uintAddress, pByte, bBit, &pStatusBits,
 &pLastStatusBit);
}
#line 444 "F:/MikroC/ModbusSlave/ModbusSlave.c"
void decodePacket(void) {
 uint uintCRCpos = 0;
 long lngCRC;

 if ( TMR0IF_bit == 1 ) {
 if ( blnCheckBuffer == FALSE && ++uintRxGap >= uintRxGapSetPt ) {
 if ( bytIndex >= 8 ) {



 blnCheckBuffer = TRUE;
 }

 uintRxGap = 0;
 }
 TMR0H =  0xfe ;
 TMR0L =  0x70 ;

 TMR0IF_bit = 0;
 }
 if( RCIF_bit ) {
 if ( bytIndex >=  255  ) {

 bytIndex = 0;
 }

 request.arybytes[bytIndex++] = (byte)UART_Read();

 startTimeout();

 RCIF_bit = 0;
 }
 if ( blnCheckBuffer == FALSE ) {

 return;
 }

 if ( request.arybytes[0] != bytSlaveAddress ) {

 blnCheckBuffer = FALSE;
 return;
 }

 uintCRCpos = 6;

 switch( request.arybytes[1] ) {
 case READ_COILS:
 case FORCE_SINGLE_COIL:
 case FORCE_MULTIPLE_COILS:
 if ( pCoils ==  ((void*)0)  ) {

 exceptionCode = ILLEGAL_DATA_ADDRESS;
 }
 break;
 case READ_STATUS_INPUTS:
 if ( pStatusBits ==  ((void*)0)  ) {

 exceptionCode = ILLEGAL_DATA_ADDRESS;
 }
 break;
 case READ_HOLDING_REGISTERS:
 case PRESET_SINGLE_REGISTER:
 case PRESET_MULTIPLE_REGISTERS:
 if ( pHoldingRegs ==  ((void*)0)  ) {

 exceptionCode = ILLEGAL_DATA_ADDRESS;
 }
 break;
 case READ_INPUT_REGISTERS:
 if ( pInputs ==  ((void*)0)  ) {

 exceptionCode = ILLEGAL_DATA_ADDRESS;
 }
 break;
 default:

 exceptionCode = ILLEGAL_FUNCTION;
 break;
 }
 bytStartHi = request.arybytes[2];
 bytStartLo = request.arybytes[3];
 bytTotalHi = request.arybytes[4];
 bytTotalLo = request.arybytes[5];

 if ( request.arybytes[1] == FORCE_MULTIPLE_COILS ||
 request.arybytes[1] == PRESET_MULTIPLE_REGISTERS ) {
 uintCRCpos += 1 + request.arybytes[6];
 }

 lngCRC = calcCRC(request.arybytes, uintCRCpos);

 if ( (lngCRC & 0xff) == request.arybytes[uintCRCpos] &&
 (lngCRC >> 0x08) == request.arybytes[uintCRCpos + 1] ) {

 uint uintSAddr, uintEAddr, uintItemCount, uintLength = 6;
 uint uintOffset, uintAddr, uintBit;
 discreteDef* pDiscretes;
 registerDef* pRegisters;

 uintSAddr = (((uint)bytStartHi << 0x08) | bytStartLo) + 1;

 uintItemCount = ((uint)bytTotalHi << 0x08) | bytTotalLo;

 uintEAddr = uintSAddr + uintItemCount - 1;

 reply.arybytes[0] = request.arybytes[0];
 reply.arybytes[1] = request.arybytes[1];

 switch( request.arybytes[1] ) {
 case READ_COILS:
 case READ_STATUS_INPUTS:
 if ( request.arybytes[1] == READ_COILS ) {
 pDiscretes = pCoils;
 } else {
 pDiscretes = pStatusBits;
 }
 if ( validDiscretes(pDiscretes, uintSAddr, uintEAddr) == FALSE ) {
 exceptionCode = ILLEGAL_DATA_ADDRESS;

 } else {
 reply.arybytes[2] = packBits(pDiscretes,
 &reply.arybytes[3],
 uintSAddr, uintEAddr);
 uintLength = reply.arybytes[2] + 3;
 }
 break;
 case READ_HOLDING_REGISTERS:
 case READ_INPUT_REGISTERS:
 case READ_STATUS_INPUTS:
 if ( request.arybytes[1] == READ_INPUT_REGISTERS ) {
 pRegisters = pInputs;
 } else {
 pRegisters = pHoldingRegs;
 }
 if ( validRegisters(pRegisters, uintSAddr, uintEAddr) == FALSE ) {

 exceptionCode = ILLEGAL_DATA_ADDRESS;
 } else {
 reply.arybytes[2] = packRegisters(pRegisters,
 &reply.arybytes[3],
 uintSAddr, uintEAddr);
 uintLength = reply.arybytes[2] + 3;
 }
 break;
 case FORCE_SINGLE_COIL:
 if ( validDiscretes(pCoils, uintSAddr, uintSAddr) == FALSE ) {

 exceptionCode = ILLEGAL_DATA_ADDRESS;
 } else {
 boolean blnOk = FALSE;

 if ( request.arybytes[4] == 0xff &&
 request.arybytes[5] == 0x0 ) {

 coilState(uintSAddr, TRUE);
 blnOk = TRUE;
 } else if ( request.arybytes[4] == 0x0 &&
 request.arybytes[5] == 0x0 ) {

 coilState(uintSAddr, FALSE);
 blnOk = TRUE;
 }
 if ( blnOk == TRUE ) {

 reply.arybytes[2] = request.arybytes[2];
 reply.arybytes[3] = request.arybytes[3];
 reply.arybytes[4] = request.arybytes[4];
 reply.arybytes[5] = request.arybytes[5];
 }
 }
 break;
 case PRESET_SINGLE_REGISTER:
 if ( validRegisters(pHoldingRegs, uintSAddr, uintSAddr) == FALSE ) {

 exceptionCode = ILLEGAL_DATA_ADDRESS;
 } else {
 setRegister(uintSAddr, request.arybytes[4], request.arybytes[5]);
 reply.arybytes[2] = request.arybytes[2];
 reply.arybytes[3] = request.arybytes[3];
 reply.arybytes[4] = request.arybytes[4];
 reply.arybytes[5] = request.arybytes[5];
 }
 break;
 case FORCE_MULTIPLE_COILS:
 if ( uintItemCount >  2040  ||
 uintItemCount > (request.arybytes[6] * 8) ) {

 exceptionCode = ILLEGAL_DATA_ADDRESS;
 break;
 }
 case PRESET_MULTIPLE_REGISTERS:
 if ( (request.arybytes[1] == PRESET_MULTIPLE_REGISTERS &&
 uintItemCount >  127 ) ) {

 exceptionCode = ILLEGAL_DATA_ADDRESS;
 } else if ( validDiscretes(pCoils,
 uintSAddr, uintEAddr) == FALSE ) {
 reply.arybytes[1] |=  0x80 ;
 } else {
 uintOffset = 0;
 uintAddr = uintSAddr;

 while( uintAddr<=uintEAddr ) {
 if ( request.arybytes[1] == PRESET_MULTIPLE_REGISTERS ) {
 setRegister(uintAddr,
 request.arybytes[7 + uintOffset],
 request.arybytes[8 + uintOffset]);
 uintOffset += 2;
 } else {
 for(uintBit=0; uintBit<8; uintBit++, uintAddr++ ) {
 coilState(uintAddr,
 request.arybytes[7 + uintOffset] & (1 << uintBit));
 }
 uintOffset++;
 }
 uintAddr++;
 }
 reply.arybytes[2] = request.arybytes[2];
 reply.arybytes[3] = request.arybytes[3];
 reply.arybytes[4] = request.arybytes[4];
 reply.arybytes[5] = request.arybytes[5];
 }
 break;
 }
 if( exceptionCode != NO_EXCEPTION ) {
 reply.arybytes[1] |=  0x80 ;
 reply.arybytes[2] = (byte)exceptionCode;
 exceptionCode = NO_EXCEPTION;
 uintLength = 3;
 }
 if ( uintLength > 0 ) {

 lngCRC = calcCRC(reply.arybytes, uintLength);
 reply.arybytes[uintLength] = (lngCRC & 0xff);
 uintLength++;
 reply.arybytes[uintLength] = (lngCRC >> 0x08);
 uintLength++;

 for( bytIndex=0; bytIndex<uintLength; bytIndex++ ) {
 UART1_Write(reply.arybytes[bytIndex]);
 }

 while( !TXSTA.TRMT ) ;
 }
 }

 request.arybytes[0] = 0;
 blnCheckBuffer = FALSE;

 bytIndex = 0;
}
#line 706 "F:/MikroC/ModbusSlave/ModbusSlave.c"
int modbusSlave(baudRate eBaud, const byte bytAddress) {
 if ( bytAddress < 1 ) {
 return -1;
 }

 memset(&request, 0, sizeof(request));

 bytSlaveAddress = bytAddress;

 uintRxGap = 0;

 exceptionCode = NO_EXCEPTION;

 switch( eBaud ) {
 case BAUD_1200:
 UART1_Init(BAUD_1200 * 100);
 uintRxGapSetPt =  825 ;
 break;
 case BAUD_2400:
 UART1_Init(BAUD_2400 * 100);
 uintRxGapSetPt =  413 ;
 break;
 case BAUD_4800:
 UART1_Init(BAUD_4800 * 100);
 uintRxGapSetPt =  206 ;
 break;
 case BAUD_9600:
 UART1_Init(BAUD_9600 * 100);
 uintRxGapSetPt =  103 ;
 break;
 case BAUD_19200:
 UART1_Init(BAUD_19200 * 100);
 uintRxGapSetPt =  52 ;
 break;
 case BAUD_38400:
 UART1_Init(BAUD_38400 * 100);
 uintRxGapSetPt =  26 ;
 break;
 case BAUD_57600:
 UART1_Init(BAUD_57600 * 100);
 uintRxGapSetPt =  17 ;
 break;
 case BAUD_115200:
 UART1_Init(BAUD_115200 * 100);
 uintRxGapSetPt =  9 ;
 break;
 default:
 return -1;
 }

 Delay_ms(100);







 T0CON.TMR0ON = 1;
 T0CON.T08BIT = 0;
 T0CON.T0CS = 0;
 T0CON.T0SE = 0;
 T0CON.PSA = 1;
 T0CON.T0PS2 = 0;
 T0CON.T0PS1 = 0;
 T0CON.T0PS0 = 0;
 TMR0H =  0xfe ;
 TMR0L =  0x70 ;

 RC1IE_bit = 1;

 RC1IF_bit = 0;

 PEIE_bit = 1;

 GIE_bit = 1;
 return 0;
}
