/**
 * File:
 *  main.c
 *
 * Notes:
 *  This application demostrates the modbus slave implementation
 *  Uses UART1 routines for communicating with serial RS232 port
 *
 * History:
 *  10/06/2012 Written by Simon Platten
 */
#include "modbusSlave.h"

static statusDef statusBits[40];
static holdingDef holdingReg[3];
static inputDef inputReg[4];
static coilDef coils[32];
/**
 * Interrupt Service Routine
 */
void interrupt() {
  decodePacket();
}
/**
 * Entry point
 */
void main() {
  int p, i;
#ifdef P18F45K22
  ANSELA = 0;
  ANSELB = 0;
  ANSELC = 0;
  ANSELD = 0;
  ANSELE = 0;
#endif
#ifdef P18F4550
  ADCON1 = 0x0f;                 // Configure all ports as digital
#endif
// Configure PORTA, PORTB, PORTC and PORTE as output
  TRISA = 0;
  TRISB = 0;
  TRISC = 0;
  TRISE = 0;
// Configure PORTD as input
  TRISD = 0xff;
// Initialise ports
  LATA = 0;
  LATB = 0;
  LATC = 0;
  LATE = 0;
// Create an instance of the modbus slave
  modbusSlave(BAUD_115200, 1);
  i = 0;
  for( p=0; p<8; p++, i++ ) {
    addCoil(&coils[i], i + 1, &LATB, p);
  }
  for( p=0; p<8; p++, i++ ) {
    addCoil(&coils[i], i + 1, &LATC, p);
  }
  for( p=0; p<8; p++, i++ ) {
    addCoil(&coils[i], i + 1, &LATE, p);
  }
  i = 0;
  for( p=0; p<8; p++, i++ ) {
    addStatusBit(&statusBits[i], i + 1, &PORTB, p);
  }
  for( p=0; p<8; p++, i++ ) {
    addStatusBit(&statusBits[i], i + 1, &PORTC, p);
  }
  for( p=0; p<8; p++, i++ ) {
    addStatusBit(&statusBits[i], i + 1, &PORTD, p);
  }
  for( p=0; p<8; p++, i++ ) {
    addStatusBit(&statusBits[i], i + 1, &PORTE, p);
  }
  addInputReg(&inputReg[0], 1, &PORTB);
  addInputReg(&inputReg[1], 2, &PORTC);
  addInputReg(&inputReg[2], 3, &PORTD);
  addInputReg(&inputReg[3], 4, &PORTE);
  addHoldingReg(&holdingReg[0], 1, &PORTB);
  addHoldingReg(&holdingReg[1], 2, &PORTC);
  addHoldingReg(&holdingReg[2], 3, &PORTE);
// Endless loop
  while (1) {
  }
}